//	Cassidy.cc

#include "Cassidy.h"
#include "CassidyConfig.h"
#include <math.h>
#include <time.h>
#include <stdlib.h>

float sign(float);
float random_float(float, float);
void hsv_to_rgb(float, float, float, rgb_color *);

CassidySaver::CassidySaver(BlanketModule *owner, BView *view, int r, bool ic, int cycle)
	: BlanketModule::Saver(owner,view,5000)
{
	srand(time(NULL));
	x = 0.5;
	y = -0.5;
	H = 0.0;
	S = 1.0;
	V = 1.0;
	t = real_time_clock();
	refresh = r;
	inColor = ic;
	inc = 360.0 / (float)cycle;
	a = random_float(-5.0,5.0);
	b = random_float(-5.0,5.0);
	c = random_float(-5.0,5.0);
}

void CassidySaver::Tick(void)
{
	float scale = 0.015;
	float w, xx;
	BPoint pt;
	BRect bounds;
	rgb_color color;
	rgb_color white = { 0xFF, 0xFF, 0xFF, 0xFF };
	uint32 tt;
	
	bounds = View()->Bounds();
	tt = real_time_clock();
	if ((tt-t) > refresh)
	{
		t = tt;
		color.red = 0x00;
		color.green = 0x00;
		color.blue = 0x00;
		a = random_float(-5.0,5.0);
		b = random_float(-5.0,5.0);
		c = random_float(-5.0,5.0);
		View()->SetHighColor(color);
		View()->FillRect(bounds);
	}
	w = (bounds.Width() < bounds.Height()) ? bounds.Width() : bounds.Height();
	pt.Set((-x-y)*w*scale+bounds.Width()/2,(-y+x)*w*scale+bounds.Height()/2);
	hsv_to_rgb(H,S,V,&color);
	if (inColor)
		View()->SetHighColor(color);
	else
		View()->SetHighColor(white);
	View()->StrokeLine(pt,pt);	
	xx = y-sign(x)*sqrt(abs(b*x+c));
	y = a-x;
	x = xx;
	H += inc;
	if (H >= 360.0)
		H -= 360.0;
	
}

Cassidy::Cassidy(BMessage *message)
	: BlanketModule(message)
{
	status_t err;
	
	err = message->FindInt32("refresh",&refresh);
	if (err != B_OK)
		refresh = 30;
	err = message->FindBool("in_color",&inColor);
	if (err != B_OK)
		inColor = false;
	err = message->FindInt32("cycle",&cycle);
	if (err != B_OK)
		cycle = 3600;
}

status_t Cassidy::Archive(BMessage *message, bool deep) const
{
	status_t err;
	
	err = BlanketModule::Archive(message,deep);
	if (err)
		return err;
	err = message->AddInt32("refresh",refresh);
	if (err)
		return err;
	err = message->AddBool("in_color",inColor);
	if (err)
		return err;
	err = message->AddInt32("cycle",cycle);
	
	return err;
}

BlanketModule::Saver *Cassidy::InstantiateSaver(BView *view, bool)
{
	return new CassidySaver(this,view,refresh,inColor,cycle);
}

BlanketModule::Config *Cassidy::InstantiateConfig(BView *view)
{
	return new CassidyConfig(this,view,refresh,inColor,cycle);
}

BArchivable *BlanketModule::Instantiate(BMessage *message)
{
	return new Cassidy(message);
}

float sign(float t)
{
	if (t > 0)
		return 1;
	else if (t < 0)
		return -1;
	else
		return 0;
}

float random_float(float min, float max)
{
	float f;
	float range = max-min;
	
	f = (float)rand() / (float)RAND_MAX;
	f = range*f + min;
	
	return f;
}

void hsv_to_rgb(float H, float S, float V, rgb_color *color)
{
	int i, r, g, b;
	float f, R, G, B;
	
	i = floor(H/60.0);
	f = fmod(H,60.0) / 60.0;
	
	switch (i)
	{
		case 0:
			R = 1;
			G = f;
			B = 0;
			break;
		case 1:
			R = 1-f;
			G = 1;
			B = 0;
			break;
		case 2:
			R = 0;
			G = 1;
			B = f;
			break;
		case 3:
			R = 0;
			G = 1-f;
			B = 1;
			break;
		case 4:
			R = f;
			G = 0;
			B = 1;
			break;
		case 5:
			R = 1;
			G = 0;
			B = 1-f;
			break;
	}
	
	r = (int)floor(V*(1+S*(R-1))*256);
	g = (int)floor(V*(1+S*(G-1))*256);
	b = (int)floor(V*(1+S*(B-1))*256);	
	color->red = (r >= 256) ? 0xFF : (uint8)r;
	color->green = (g >= 256) ? 0xFF : (uint8)g;
	color->blue = (b >= 256) ? 0xFF : (uint8)b;
	color->alpha = 0xFF;
}
