//	CassidyConfig.cc

#include "CassidyConfig.h"
#include "Cassidy.h"

enum
{
	CC_SET_REDRAW = 'ccSR',
	CC_SET_COLOR = 'ccSC',
	CC_CYCLE_COLOR = 'ccCC'
};

CassidyFilter::CassidyFilter(CassidyConfig *object)
	: BMessageFilter(B_ANY_DELIVERY,B_ANY_SOURCE)
{
	config = object;
}

filter_result CassidyFilter::Filter(BMessage *message, BHandler **handler)
{
	Cassidy *owner = (Cassidy *)config->Owner();
	
	switch (message->what)
	{
		case CC_SET_REDRAW:
			owner->refresh = config->redrawSlider->Value();
			break;
		case CC_SET_COLOR:
			if (config->colorCheck->Value() == B_CONTROL_ON)
			{
				owner->inColor = true;
				config->colorSlider->SetEnabled(true);
			}
			else
			{
				owner->inColor = false;
				config->colorSlider->SetEnabled(false);
			}
			break;
		case CC_CYCLE_COLOR:
			owner->cycle = config->colorSlider->Value();
			break;
	}
}

CassidyConfig::CassidyConfig(BlanketModule *owner, BView *view, int refresh, bool inColor, int cycle)
	: BlanketModule::Config(owner,view,true)
{
	BFont font;
	BStringView *label;
	BRect rect, bounds;
	font_height fontHeight;
	BWindow *window = view->Window();
	
	if (window)
		window->Lock();
	font.SetSize(14);
	font.GetHeight(&fontHeight);
	bounds = view->Bounds();
	rect = bounds;
	rect.bottom = rect.top+fontHeight.ascent+fontHeight.descent+fontHeight.leading;
	label = new BStringView(rect,NULL,"Cassidy");
	label->SetFont(&font);
	label->SetAlignment(B_ALIGN_CENTER);
	view->AddChild(label);
	rect.top = rect.bottom+1;
	font.SetSize(9);
	font.GetHeight(&fontHeight);
	rect.bottom = rect.top+fontHeight.ascent+fontHeight.descent+fontHeight.leading;
	label = new BStringView(rect,NULL,"version 1.0");
	label->SetFont(&font);
	label->SetAlignment(B_ALIGN_CENTER);
	view->AddChild(label);
	rect.top = rect.bottom+1;
	font.SetSize(10);
	font.GetHeight(&fontHeight);
	rect.bottom = rect.top+fontHeight.ascent+fontHeight.descent+fontHeight.leading;
	label = new BStringView(rect,NULL,"by Ernest Tomlinson");
	label->SetFont(&font);
	label->SetAlignment(B_ALIGN_CENTER);
	view->AddChild(label);
	rect.top = rect.bottom+2;
	rect.bottom = rect.top;
	redrawSlider = new BSlider(rect,NULL,"Draw a new pattern...",new BMessage(CC_SET_REDRAW),3,150);
	redrawSlider->SetValue(refresh);
	redrawSlider->SetTarget(view);
	redrawSlider->SetHashMarkCount(8);
	redrawSlider->SetHashMarks(B_HASH_MARKS_BOTTOM);
	redrawSlider->SetLimitLabels("Often","Seldom");
	redrawSlider->ResizeToPreferred();
	view->AddChild(redrawSlider);
	rect.top = rect.bottom+redrawSlider->Bounds().Height()+2;
	rect.bottom = rect.top;
	colorCheck = new BCheckBox(rect,NULL,"Draw in Color",new BMessage(CC_SET_COLOR));
	colorCheck->SetValue(inColor ? B_CONTROL_ON : B_CONTROL_OFF);
	colorCheck->SetTarget(view);
	colorCheck->ResizeToPreferred();
	view->AddChild(colorCheck);
	rect.top = rect.bottom+colorCheck->Bounds().Height()+2;
	rect.bottom = rect.top;
	rect.left += 10;
	rect.right -= 10;
	colorSlider = new BSlider(rect,NULL,"Cycle through colors...",new BMessage(CC_CYCLE_COLOR),360,48000);
	colorSlider->SetValue(cycle);
	colorSlider->SetTarget(view);
	colorSlider->SetHashMarkCount(8);
	colorSlider->SetHashMarks(B_HASH_MARKS_BOTTOM);
	colorSlider->SetLimitLabels("Quickly","Glacially");
	colorSlider->ResizeToPreferred();
	view->AddChild(colorSlider);
	colorSlider->SetEnabled(inColor);
	rect.top = rect.bottom+colorSlider->Bounds().Height()+2;
	rect.bottom = bounds.bottom;
	view->AddChild(GetPreview(rect,B_FOLLOW_ALL_SIDES,B_WILL_DRAW));
	view->AddFilter(new CassidyFilter(this));
	if (window)
		window->Unlock();
}
